-- Lightroom SDK
local LrLogger = import "LrLogger"
local LrDialogs	= import "LrDialogs"
local LrFileUtils	= import "LrFileUtils"
local LrApplication	= import "LrApplication"
local LrApplicationView = import "LrApplicationView"
local LrTasks	= import 'LrTasks'
local LrPathUtils	= import 'LrPathUtils'
local LrStringUtils	= import 'LrStringUtils'
local LrDate = import 'LrDate'
local LrDialogs = import 'LrDialogs'

require 'PluginData'
require 'CatalogDeletion'


local pathToPhotoMap
local shouldfindByPath = false
local lastImportPhoto = nil


-- function needed for brut force searching in LR Lib 
function tryAccessCatalog(fn, ...)
  local status, data = LrTasks.pcall(fn, unpack(arg))
  if status == true then
    return data
  else
    logger:trace("Cannot retrieve photos from the Lightroom catalog: " .. data)
    error("Cannot retrieve data from the Lightroom catalog. The catalog might be corrupted.")
  end
end


-- find a photo in a catalog within its internal LR ID
function findPhotoByIdInCatalog(catalog, photoID)
  logger:trace("searching for id: '" .. photoID .. "'")
  local photo = catalog:findPhotoByUuid(photoID)
  if photo ~= nil then
    logger:trace(".. found ")
    return photo
  end
end
-- select the selected Photo in catalog
function selectPhotoAsync(catalog, photo)
  LrTasks.startAsyncTask( 
    function()  
        catalog:setSelectedPhotos(photo,{})
    end)
end

-- check if the photo is selected in catalog
-- if not, reselect it. 
function checkPhotoIsSelected(catalog, photo)
  LrTasks.startAsyncTask( 
    function(context)  
      local selectedPhoto = catalog:getTargetPhoto()
      if photo ~= selectedPhoto then
        logger:trace("should reselect Photo !")
        selectPhotoAsync(catalog, photo)
      end
    end)
end

-- select a photo and retry if needed
function selectPhotoAndRetry(catalog,photo)
  -- change activeView Before
  local result = catalog:setActiveSources(catalog.kAllPhotos)  
  logger:trace("showing photo")
  selectPhotoAsync(catalog, photo)
  LrApplicationView.showView("loupe")
  checkPhotoIsSelected(catalog, photo)
  return true
end

-- open a photo in active catalog 
-- param: uuid of the pict
function openPhoto(catalog, uuid)
  local photo = tryAccessCatalog(findPhotoByIdInCatalog, catalog, uuid)
  if photo ~= nil then
    return selectPhotoAndRetry(catalog,photo)
  else
    logger:trace("can't find photo " .. uuid)
    return false
  end
end

-- create a preview for a list of photo
function createPreviewFor(photos) 
  for i, photo in ipairs(photos) do 
    logger:trace("will create Preview fo Photo: " .. photo.localIdentifier)
    local request = photo:requestJpegThumbnail( 4096, 4096, function(img)
      if img == nil  then 
        logger:error("Error during creation of full res preview")
      else
        logger:trace("preview created")
      end
    end )
  end
end

-- create a preview in LR for this version
-- TODO: add size of the preview
function createPreview(catalog, uuid)
  local photo = tryAccessCatalog(findPhotoByIdInCatalog, catalog, uuid) 
  logger:trace("will create Preview fo Photo: " .. uuid)
  if photo ~= nil then
    local photos = {photo}
    createPreviewFor(photos)
  end
  logger:trace("end createPreview".. uuid)
end

-- import a file in the catalog
function importFile(catalog, path)
  logger:trace("will import one file at '".. path .."'")
  catalog:withWriteAccessDo("Import file",function(context)
    lastImportPhoto = catalog:addPhoto(path)
  end
  )
end

-- try to process the file receive by Peakto
-- this method is called with a file containing Instructions for the plugin 
function tryToProcessFile(fileName)
  logger:trace("Process started for file: " .. fileName)
  if LrFileUtils.exists(fileName) then
    local task = LrTasks.startAsyncTask( 
      function(context)
        local fileData = LrFileUtils.readFile(fileName)
        local fileDataLen = string.len(fileData)
        local _, numberOfLines = string.gsub(fileData, "\n", "\n")
        local lineFrom = 1
        local catalog = LrApplication.activeCatalog()
        -- an instance of CatalogDeletion
        local catalogDeletion = CatalogDeletion:new()
        repeat -- read lines in file
          local lineTo = string.find(fileData, "\n", lineFrom)
          if lineTo ~= nil then            
            logger:trace("lineFrom: " .. lineFrom .. ", lineTo: " .. lineTo)
            local currentLine = string.sub(fileData, lineFrom, lineTo)
            logger:trace("currentLine: " .. currentLine)
            if string.len(currentLine) > 12 then 
              -- local fullCommand = LrStringUtils.trimWhitespace(currentLine)
              local command = string.sub(currentLine, 1, 12) 
              local uuid = string.sub(currentLine, 13, string.len(currentLine))
              local photoUuid = LrStringUtils.trimWhitespace(uuid)
              logger:trace("Receive Command " .. command .. " for '" .. photoUuid .. "'")
              if command == "OPENVERSION:" then 
                if openPhoto(catalog,photoUuid) then 
                  logger:trace("command done")
                end
              end
              if command == "CREAPREVIEW:" then
                createPreview(catalog, photoUuid)
              end
              if command == "IMPORTFILES:" then 
                importFile(catalog, photoUuid)
              end
              if command == "ADDDELFILES:" then 
                catalogDeletion:add(photoUuid)
              end
              if command == "PROCESSDELT:" then
                catalogDeletion:process(catalog)
              end
            end
            lineFrom = lineTo + 1
          end
        until(nil == lineTo)
        if lastImportPhoto ~= nil then 
          selectPhotoAndRetry(catalog, lastImportPhoto)
          lastImportPhoto = nil
        end
        LrFileUtils.delete(fileName)
        pathToPhotoMap = nil
      end
    )
  end
end


-- MAIN Process
-- When Plugin start
logger = LrLogger(PluginData.peaktoLoggerFileName)
if PluginData.peaktoUseLogging then
  logger:enable("logfile")
end
logger:trace("======= PluginInit Loaded " .. os.date() .. " =======")

-- Get the path to the user directory
local containerDirectory = LrPathUtils.standardizePath( '~/Library/Containers' )

-- when lauching, peakto create an indicator file in its temps directory in order to see if it is a sandboxed Version
baseTempDirPath = LrPathUtils.getStandardFilePath('temp')
basePeaktoTempDir = LrPathUtils.child(containerDirectory, 'io.cyme.Peakto/Data/tmp')
basePeakoMasTempDir = LrPathUtils.child(containerDirectory,'io.cyme.PeaktoMAS/Data/tmp')

logger:trace("BaseTempDir: " .. baseTempDirPath)

local standardTempDirPath = baseTempDirPath
indicatorFileName = LrPathUtils.child(baseTempDirPath, PluginData.peaktoIndicatorFileName)
logger:trace("test: " .. indicatorFileName)

-- Now we try to find where Peakto leave its instructions.
if not LrFileUtils.exists(indicatorFileName) then 
  -- peakto sandboxed in container
  standardTempDirPath = basePeaktoTempDir
  indicatorFileName = LrPathUtils.child(basePeaktoTempDir, PluginData.peaktoIndicatorFileName)
  logger:trace("test: " .. indicatorFileName)

  if not LrFileUtils.exists(indicatorFileName) then 
    --peakto MAS in container
    standardTempDirPath = basePeakoMasTempDir
    indicatorFileName = LrPathUtils.child(basePeakoMasTempDir, PluginData.peaktoIndicatorFileName)
    logger:trace("test: " .. indicatorFileName)
  end

  if not LrFileUtils.exists(indicatorFileName) then 
    -- peakto sandboxed in /var/tmp/ folder
    standardTempDirPath = LrPathUtils.child(baseTempDirPath,"io.cyme.Peakto")
    indicatorFileName = LrPathUtils.child(standardTempDirPath, PluginData.peaktoIndicatorFileName)
    logger:trace("test: " .. indicatorFileName)
  end

  if not LrFileUtils.exists(indicatorFileName) then 
    -- peakto Mas in /var/tmp/ folder
    standardTempDirPath = LrPathUtils.child(baseTempDirPath,"io.cyme.PeaktoMAS")
    indicatorFileName = LrPathUtils.child(standardTempDirPath, PluginData.peaktoIndicatorFileName)
    logger:trace("test: " .. indicatorFileName)
  end

end

if not LrFileUtils.exists(indicatorFileName) then 
  -- nothing found, we go to standard location
  standardTempDirPath = basePeaktoTempDir
end
 

local importFileName = LrPathUtils.child(standardTempDirPath, PluginData.peaktoTempImportFileName)
local triggerFileName = LrPathUtils.child(standardTempDirPath, PluginData.peaktoTriggerFileName)

logger:trace("Looking for " .. triggerFileName)
logger:trace("To read commands in file: " .. importFileName)

if LrFileUtils.exists(triggerFileName) then
  LrFileUtils.delete(triggerFileName)
  tryToProcessFile(importFileName)
else 
  -- in case of a relaunch (open another catalog)
  if LrFileUtils.exists(importFileName) then 
    tryToProcessFile(importFileName)
  end
end

-- pluginIsRunning is used since async task can be running even after plug-in shutdown
pluginIsRunning = 1
LrTasks.startAsyncTask( function()
  while pluginIsRunning == 1 do
    if LrFileUtils.exists(triggerFileName) then
      LrFileUtils.delete(triggerFileName)
      tryToProcessFile(importFileName)
    end
    --sleep for 1 second
    LrTasks.sleep(1)
  end
end)
